@@ -1,3 +1,26 @@
+#
+# Basic BioStudio functions
+#
+# POD documentation - main docs before the code
+
+=head1 NAME
+
+BioStudio::Cairo
+
+=head1 VERSION
+
+Version 1.04
+
+=head1 DESCRIPTION
+
+BioStudio functions for Cairo interaction.
+
+=head1 AUTHOR
+
+Sarah Richardson <notadoctor@jhu.edu>.
+
+=cut
+
 package Bio::BioStudio::Cairo;
 require Exporter;
 
@@ -12,10 +35,10 @@ use strict;
 use warnings;
 
 use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);
-$VERSION = '1.03';
+$VERSION = '1.04';
 
 @ISA = qw(Exporter);
-@EXPORT_OK = qw(  
+@EXPORT_OK = qw( 
   draw_scale
   draw_feature
   draw_RE
@@ -28,19 +51,19 @@ $VERSION = '1.03';
   draw_amplicon
   draw_repeats
   draw_UTC
+  draw_deletion
 );
 %EXPORT_TAGS = (
-  all => [qw(draw_scale draw_RE draw_stop draw_ARS draw_SSTR draw_feature 
-          draw_centromere draw_CDS draw_intron draw_amplicon draw_repeats
-          draw_UTC)],
-  basic => [qw(draw_scale draw_feature)]
-  );
+  all => \@EXPORT_OK, basic => [qw(draw_scale draw_feature)]);
 
 my %featsize = ("site_specific_recombination_target_region" => 50,
       "stop_retained_variant" => 20);
-################################################################################
-########################### Cairo Drawing  Functions ###########################
-################################################################################
+
+=head1 Functions
+
+=head2 draw_scale
+
+=cut
 
 sub draw_scale
 {
@@ -75,14 +98,6 @@ sub draw_scale
       $ctx->move_to($j - $thref->{width} / 2, $pa->{SCALE_HEIGHT} + $i * $pa->{LEVEL_HEIGHT} - ($bigScaleMarkHeight + 10));
       $ctx->show_text (int($bigScaleMarkText / 1000) . " kb");
       $bigScaleMarkText += $bigScaleMark * $pa->{FACTOR};
-
-      #comment out after debugging
-      for (my $k = $j; $k <= $j + $bigScaleMark; $k+= int(1000/$pa->{FACTOR}))
-      {
-        #$ctx->move_to($k, $pa->{SCALE_HEIGHT} + $i * $pa->{LEVEL_HEIGHT});
-        #$ctx->rel_line_to(0, -50);
-        #$ctx->stroke();
-      }
     }
     $bigScaleMarkText -= $bigScaleMark * $pa->{FACTOR};
     if ($i == $totalLevel)
@@ -102,164 +117,207 @@ sub draw_scale
   return $totalLevel;
 }
 
+=head2 draw_feature
+
+=cut
+
 sub draw_feature
 {
-  my ($ctx, $obj, $xbeg, $xend, $isdash, $pa) = @_;
+  my ($ctx, $obj, $xbeg, $xend, $isdash, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   $ctx->set_dash(15,7,7) if $isdash;
   $ctx->set_dash(0) unless $isdash;
-  my $y = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
-  my $flag;
-  my $clipEnd = $obj->{DrawEnd};
-  if ($clipEnd >= $xend)
+  $xywref = $xywref ? $xywref : undef;
+  unless ($xywref)
   {
-    $clipEnd = $xend;
-    $flag = 1;
+    my $flag;
+    my $clipEnd = $obj->{DrawEnd};
+    if ($clipEnd >= $xend)
+    {
+      $clipEnd = $xend + 2;
+      $flag = 1;
+    }
+    my $clipStart = $obj->{DrawStart}; 
+    if ($clipStart <= $xbeg)
+    {
+       $clipStart = $xbeg - 2;
+       $flag = 1;
+    }
+    my $clipWidth = $clipEnd - $clipStart;
+    if (exists $featsize{$feat->primary_tag})
+    {
+      $clipWidth = $featsize{$feat->primary_tag};
+      $clipStart -= .5 *$clipWidth;
+    }
+    if ($flag)
+    {
+      ##I WISH THIS CLIPPING REGION WAS BETTER RESTRICTED TO OBJECT HEIGHT
+      my $y = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+      $ctx->rectangle($clipStart, $y, $clipWidth, $pa->{LEVEL_HEIGHT});
+      $ctx->clip();   
+    }   
   }
-  my $clipStart = $obj->{DrawStart};  
-  if ($clipStart <= $xbeg)
-  {
-     $clipStart = $xbeg;
-     $flag = 1;
-  }
-  my $clipWidth = $clipEnd - $clipStart;
-  if (exists $featsize{$feat->primary_tag})
-  {
-    $clipWidth = $featsize{$feat->primary_tag};
-    $clipStart -= .5 *$clipWidth;
-  }
-  if ($flag)
-  {
-    $ctx->rectangle($clipStart, $y, $clipWidth, $pa->{LEVEL_HEIGHT});
-    #print "\t drawing a $isdash $clipWidth wide rectangle at ($clipStart - $clipEnd), $y for ", 
-     #         $feat, " from ", $obj->{DrawStart}, "..", $obj->{DrawEnd}, " ($xbeg..$xend)\n";
-    $ctx->clip();    
-  }
-  
-  #Check what feature is it and start drawing (Put it in alphabetical order) 
+ 
+  #Check what feature is it and start drawing (Put it in alphabetical order)
   if ($feat->primary_tag eq "CDS")
   {
-    draw_CDS($ctx, $obj, $pa);
+    draw_CDS($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "PCR_product")
   {
-    draw_amplicon($ctx, $obj, $pa);
+    draw_amplicon($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "site_specific_recombination_target_region")
   {
-    draw_SSTR($ctx, $obj, $pa);
+    draw_SSTR($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "ARS")
   {
-    draw_ARS($ctx, $obj, $pa);
+    draw_ARS($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "stop_retained_variant")
   {
-    draw_stop($ctx, $obj, $pa);
+    draw_stop($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "centromere")
   {
-    draw_centromere($ctx, $obj, $pa);
+    draw_centromere($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "restriction_enzyme_recognition_site")
   {
-    draw_RE($ctx, $obj, $pa);
+    draw_RE($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "intron")
   {
-    draw_intron($ctx, $obj, $pa);
+    draw_intron($ctx, $obj, $pa, $xywref);
   }
   elsif ($feat->primary_tag eq "repeat_family")
   {
-    draw_repeats($ctx, $obj, $pa);
-  }  
+    draw_repeats($ctx, $obj, $pa, $xywref);
+  } 
   elsif ($feat->primary_tag eq "universal_telomere_cap")
   {
-    draw_UTC($ctx, $obj, $pa);
-  }  
+    draw_UTC($ctx, $obj, $pa, $xywref);
+  }
+  elsif ($feat->primary_tag eq "deletion")
+  {
+    draw_deletion($ctx, $obj, $pa, $xywref);
+  } 
   #Reset all drawing setting
   $ctx->set_dash(0);
   $ctx->reset_clip();
 }
 
+=head2 draw_RE
+
+=cut
+
 sub draw_RE
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
-  
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgba($colref->[0], $colref->[1], $colref->[2], .95);
-  
-  my $RELineHeight = 50.0;  
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $radius = ($end-$start)/2;
-  $ctx->move_to($start + $radius, $pa->{SCALE_HEIGHT} + $obj->{LevelNum}*$pa->{LEVEL_HEIGHT});
-  $ctx->rel_line_to(0, -$RELineHeight);
+ 
+  my $RELineHeight = 50.0;
+  my ($start, $radius, $movey);
   my $thref = ($ctx->text_extents($feat->Tag_enzyme));
-  my $tx = $start-$thref->{width}/2;
-  my $ty = $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT} - ($RELineHeight+10);
+  unless ($xywref)
+  {
+    $start = $obj->{DrawStart};
+    $radius = ($obj->{DrawEnd} - $start) / 2;
+    $movey = $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+  }
+  else
+  {
+    ($start, $radius, $movey) = @$xywref;
+    $radius = $radius / 2;
+  } 
+  $ctx->move_to($start + $radius, $movey);
+  $ctx->rel_line_to(0, -$RELineHeight);
+ 
+  my $tx = $start - $thref->{width}/2;
+  my $ty = $movey - ($RELineHeight+10);
   $ctx->move_to($tx, $ty);
   $ctx->show_text ($feat->Tag_enzyme);
   $ctx->fill_preserve();
   $ctx->set_source_rgb(0, 0, 0);
   $ctx->stroke();
-  return; 
+  return;
 }
 
+=head2 draw_centromere
+
+=cut
+
 sub draw_centromere
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
-  #print "\t\tDrawing " . $feat->Tag_load_id, "\n";
-  
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgb($colref->[0], $colref->[1], $colref->[2]);
-  
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $radius = ($end-$start) / 2;
-  my $ypos = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
-    #Centromere radius
+ 
+  my ($start, $movey, $radius);
+  unless ($xywref)
+  {
+    $start = $obj->{DrawStart};
+    $radius = ($obj->{DrawEnd} - $start) / 2;
+    $movey = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+    $movey += $pa->{STRAND_Y_POS} + $pa->{STRAND_DISTANCE}/2;
+  }
+  else
+  {
+    ($start, $movey, $radius) = @$xywref;
+    $radius = $radius / 2;
+  }
   my $centromereRadius = 200/$pa->{FACTOR};
   $centromereRadius = $radius if ($radius > $centromereRadius);
   $centromereRadius = 7 if ($centromereRadius < 7);
-  
-  #Draw Circle Shape
-  $ctx->arc($start + $radius, 
-            $pa->{STRAND_Y_POS} + $pa->{STRAND_DISTANCE}/2 + $ypos, 
-            $centromereRadius, 
-            0, 
-            2*pi);
+ 
+  $ctx->move_to($start, $movey);
+  $ctx->arc($start + $radius, $movey, $centromereRadius, 0, 2*pi);
   $ctx->fill_preserve();
   $ctx->set_source_rgb(0, 0, 0);
   $ctx->stroke();
 }
 
+=head2 draw_stop
+
+=cut
+
 sub draw_stop
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   my $parent = $feat->Tag_parent_id;
   my $pobj = $pa->{FEATURES}->{$parent};
   my $pfeat = $pobj->{feat};
-  #print "\t\tDrawing " . $feat->Tag_load_id, "\n";
-  
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgb($colref->[0], $colref->[1], $colref->[2]);
-  
-  #Preset Size of the Diamond
+ 
   my $CodonSide = 80 / $pa->{FACTOR};
   $CodonSide = 4 if ($CodonSide < 4);
   my $smove = $CodonSide / sqrt(2);
-  
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $radius = ($end-$start)/2;
-  ($start,$end) = ($end, $start) if ($pfeat->strand == -1);
-  my $ypos = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
-  my ($xmove, $ymove) = ($start + $radius, $pa->{STRAND_Y_POS} + $ypos);
-  $ymove += $pa->{STRAND_DISTANCE} if ($pfeat->strand == -1);
-  $ctx->move_to($xmove, $ymove);
-  
-  #Draw diamond octagon
+ 
+  my ($start, $movey, $radius);
+  unless ($xywref)
+  {
+    $start = $obj->{DrawStart};
+    $radius = ($obj->{DrawEnd} - $start) / 2;
+    $start = $obj->{DrawEnd} if ($pfeat->strand == -1);
+    $movey = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT} + $pa->{STRAND_Y_POS};
+    $movey += $pa->{STRAND_DISTANCE} if ($pfeat->strand == -1);   
+  }
+  else
+  {
+    ($start, $movey, $radius) = @$xywref;
+    $radius = $radius/2;
+  }
+  $ctx->move_to($start + $radius, $movey);
+
   $ctx->rel_move_to(-$CodonSide / 2, -($CodonSide / 2+$smove));
   $ctx->rel_line_to($CodonSide, 0);
   $ctx->rel_line_to($smove, $smove);
@@ -274,47 +332,72 @@ sub draw_stop
   $ctx->stroke();
 }
 
+=head2 draw_ARS
+
+=cut
+
 sub draw_ARS
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
-#  print "\t\tDrawing " . $feat->Tag_load_id, "\n";
 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgba($colref->[0], $colref->[1], $colref->[2], .95);
   my $ARSHeight = 50;
 
-  #Draw Block Shape
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $ypos = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
-  $ctx->move_to($start, $pa->{SCALE_HEIGHT}+$ypos);
+  my ($start, $movey, $width);
+  unless($xywref)
+  {
+    $start = $obj->{DrawStart};
+    $width = $obj->{DrawEnd} - $start;
+    $movey = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT} + $pa->{SCALE_HEIGHT};   
+  }
+  else
+  {
+    ($start, $movey, $width) = @$xywref;
+  }
+  $ctx->move_to($start, $movey);
+ 
   $ctx->rel_line_to(0, -$ARSHeight/2);
-  $ctx->rel_line_to($end-$start, 0);
+  $ctx->rel_line_to($width, 0);
   $ctx->rel_line_to(0, $ARSHeight);
-  $ctx->rel_line_to(-($end-$start), 0);
+  $ctx->rel_line_to(-$width, 0);
   $ctx->close_path();
   $ctx->fill_preserve();
   $ctx->set_source_rgb(0, 0, 0);
   $ctx->stroke();
 }
 
+=head2 draw_SSTR
+
+=cut
+
 sub draw_SSTR
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
-  #print "\t\tDrawing " . $feat->Tag_load_id, "\n";
-  
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgba($colref->[0], $colref->[1], $colref->[2], .95);
-             
-  #Draw diamond sharp
+            
   my $SSRTsize = $pa->{U_BIG_SCALE_MARK} / ($pa->{FACTOR} * 15);
   $SSRTsize = 7 if ($SSRTsize < 7);
   $SSRTsize = 20 if ($SSRTsize > 20);
-  
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $ypos = $obj->{LevelNum}*$pa->{LEVEL_HEIGHT};
-  $ctx->move_to($start+($end-$start)/2, $pa->{SCALE_HEIGHT}+$ypos-$SSRTsize);
+ 
+  my ($start, $movey, $radius);
+  unless ($xywref)
+  {
+    $start = $obj->{DrawStart};
+    $radius =  ($obj->{DrawEnd} - $start) / 2;
+    $movey = $obj->{LevelNum}*$pa->{LEVEL_HEIGHT} + $pa->{SCALE_HEIGHT};
+  }
+  else
+  {
+    ($start, $movey, $radius) = @$xywref;
+    $radius = $radius / 2;
+  }
+  $ctx->move_to($start + $radius, $movey - $SSRTsize);
+ 
   $ctx->rel_line_to($SSRTsize, $SSRTsize);
   $ctx->rel_line_to(-$SSRTsize, $SSRTsize);
   $ctx->rel_line_to(-$SSRTsize, -$SSRTsize);
@@ -324,16 +407,17 @@ sub draw_SSTR
   $ctx->stroke();
 }
 
+=head2 draw_CDS
+
+=cut
+
 sub draw_CDS
 {
-  my ($ctx, $obj, $pa) = @_;
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   my $triLen = 50;
   my $CDSHeight = 50;
-  #$ctx->rectangle()
-  #print "\t\tDrawing " . $feat->Tag_load_id, "\n";
-  #Set Color
+ 
   my $key = $feat->Tag_essential_status eq "Essential"
           ? "Essential"
           : $feat->Tag_essential_status eq "fast_growth"
@@ -341,35 +425,43 @@ sub draw_CDS
             : $feat->Tag_orf_classification;
   my $colref = $pa->{FEAT_RGB}->{$key};
   $ctx->set_source_rgb($colref->[0], $colref->[1], $colref->[2]);
-  
-  #inital the drawing by identify which direction is processing and move to the start point
-  my ($smove, $emove);
-  if ($feat->strand == 1)
+ 
+  my ($start, $end, $movey, $width);
+  unless ($xywref)
   {
-    ($smove, $emove) = ($start, $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT});
+    $start = $obj->{DrawStart};
+    $end = $obj->{DrawEnd};
+    $width = $end - $start;
+    $movey = $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+    if ($feat->strand == -1)
+    {
+      ($start, $end) = ($end, $start);
+      $movey += $pa->{STRAND_DISTANCE};
+      my $unitVec = (($end-$start) / abs($end-$start));
+      $triLen = $triLen * $unitVec;
+    }   
   }
-  elsif ($feat->strand == -1)
-  {
-    ($start, $end) = ($end, $start);
-    ($smove, $emove) = ($start, $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT} + $pa->{STRAND_DISTANCE});
-    my $unitVec = (($end-$start) / abs($end-$start));
-    $triLen = $triLen * $unitVec;
-  }  
-  #Calculate x2 = rectangle's Len , x3 = triangle part's Len |----x2----->
-  my ($x2, $x3);
-  if (abs($end-$start) <= abs($triLen * 1.3))
-  {   
-    $x2 = $end-($end-$start)/2-$start;
-    $x3 = ($end-$start)/2;
-  } 
   else
   {
-    $x2 = $end-$triLen-$start;
+    ($start, $movey, $width) = @$xywref;
+    $end = $width + $start;
+  }
+
+  #Calculate x2 = rectangle's Len , x3 = triangle part's Len |----x2----->
+  my ($x2, $x3);
+  if ($width <= abs($triLen * 1.3))
+  {  
+    $x2 = $end - ($end - $start) / 2-$start;
+    $x3 = ($end-$start)/2;
+  }
+  else
+  {
+    $x2 = $end - $triLen - $start;
     $x3 = $triLen;
   }
-  $pa->{CDSDATA}->{$feat->Tag_parent_id} = [$smove, $emove, $x2, $x3];
-  $ctx->move_to($smove, $emove);
-  
+  $pa->{CDSDATA}->{$feat->Tag_parent_id} = [$start, $movey, $x2, $x3];
+  $ctx->move_to($start, $movey);
+ 
   $ctx->rel_line_to(0, 0-$CDSHeight/2);
   $ctx->rel_line_to($x2, 0);
   $ctx->rel_line_to($x3, $CDSHeight/2);
@@ -381,112 +473,166 @@ sub draw_CDS
   $ctx->stroke();
 }
 
+=head2 draw_intron
+
+=cut
+
 sub draw_intron
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   my $parent = $feat->Tag_parent_id;
   my $pobj = $pa->{FEATURES}->{$parent};
   my $pfeat = $pobj->{feat};
-  my ($triLen, $CDSHeight) = (50, 50);
-  
+  my $CDSHeight = 50;
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgb($colref->[0], $colref->[1], $colref->[2]);
-    
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $radius = ($end-$start)/2;
-  my $movey = $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
-  $movey+= $pa->{STRAND_DISTANCE} if ($pfeat->strand == -1);
+ 
+  my ($start, $movey, $radius);
+  unless ($xywref)
+  {
+    $start = $obj->{DrawStart};
+    $radius = ($obj->{DrawEnd} - $start) / 2;
+    $movey = $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+    $movey += $pa->{STRAND_DISTANCE} if ($pfeat->strand == -1);   
+  }
+  else
+  {
+    ($start, $movey, $radius) = @$xywref;
+  }
   $ctx->move_to($start, $movey);
 
-  #Draw two Line
   $ctx->rel_line_to($radius, -$CDSHeight/3.0);
   $ctx->rel_line_to($radius, $CDSHeight/3.0);
   $ctx->fill_preserve();
+ 
   $ctx->set_source_rgb(0, 0, 0);
   $ctx->stroke();
 }
 
+=head2 draw_amplicon
+
+=cut
+
 sub draw_amplicon
 {
-  my ($ctx, $obj, $pa) = @_;
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   my $gene = $feat->Tag_ingene;
-  #print "\t\tDrawing " . $feat->Tag_load_id, "\n";
-  
+  my $CDSHeight = 50;
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
-  $ctx->set_source_rgba($colref->[0], $colref->[1], $colref->[2], .5);
-  
-  #Use Part of cDS drawing part to get the exact shape
-  my ($triLen, $CDSHeight) = (50, 50);
-  
-  my ($psmove, $pemove, $px2, $px3) = @{$pa->{CDSDATA}->{$feat->Tag_ingene}};
-  my ($smove, $emove) = ($start, $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT});
-  $emove+= $pa->{STRAND_DISTANCE} if ($feat->strand == -1);
-  my $mwidth = abs($end-$start);
+  $ctx->set_source_rgba($colref->[0], $colref->[1], $colref->[2], .8);
+ 
+  my ($start, $movey, $mwidth);
+ 
+  unless ($xywref)
   {
-    $ctx->move_to($psmove, $pemove);
-    $ctx->rel_line_to(0, 0-$CDSHeight/2);
-    $ctx->rel_line_to($px2, 0);
-    $ctx->rel_line_to($px3, $CDSHeight/2);
-    $ctx->rel_line_to(0-$px3, $CDSHeight/2);
-    $ctx->rel_line_to(0-$px2, 0);
-    $ctx->clip();
+    $start = $obj->{DrawStart};
+    $mwidth = abs($obj->{DrawEnd}-$start);
+    my ($psmove, $pemove, $px2, $px3) = @{$pa->{CDSDATA}->{$feat->Tag_ingene}};
+    {
+      $ctx->move_to($psmove, $pemove);
+      $ctx->rel_line_to(0, 0-$CDSHeight/2);
+      $ctx->rel_line_to($px2, 0);
+      $ctx->rel_line_to($px3, $CDSHeight/2);
+      $ctx->rel_line_to(0-$px3, $CDSHeight/2);
+      $ctx->rel_line_to(0-$px2, 0);
+      $ctx->clip();
+    }   
+    $movey = $pa->{STRAND_Y_POS} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+    $movey += $pa->{STRAND_DISTANCE} if ($feat->strand == -1);
   }
-  $ctx->move_to($smove, $emove);
+  else
+  {
+    ($start, $movey, $mwidth) = @$xywref;
+  }
+  $ctx->move_to($start, $movey);
+ 
   $ctx->rel_line_to(0, $CDSHeight/2);
   $ctx->rel_line_to($mwidth, 0);
   $ctx->rel_line_to(0, -$CDSHeight);
-  $ctx->rel_line_to(- $mwidth, 0); 
+  $ctx->rel_line_to(- $mwidth, 0);
   $ctx->close_path();
   $ctx->fill_preserve();
   $ctx->set_source_rgb(0, 0, 0);
   $ctx->stroke();
 }
 
+=head2 draw_repeats
+
+=cut
+
 sub draw_repeats
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   my $repeatfamilyHeight = 50;
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgba($colref->[0], $colref->[1], $colref->[2], .95);
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $width = $end - $start;    
-  $ctx->move_to($start, $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT});
+ 
+  my ($start, $width, $movey);
+  unless ($xywref)
+  { 
+    $start = $obj->{DrawStart};
+    my $end = $obj->{DrawEnd};
+    $width = $end - $start;
+    $movey = $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+  }
+  else
+  {
+    ($start, $movey, $width) = @$xywref;
+  }
+  $ctx->move_to($start, $movey);
+ 
   $ctx->rel_line_to(0, -$repeatfamilyHeight/2);
   $ctx->rel_line_to($width, 0);
   $ctx->rel_line_to(0, $repeatfamilyHeight);
   $ctx->rel_line_to(-$width, 0);
   $ctx->close_path();
+ 
   $ctx->fill_preserve();
   $ctx->set_source_rgb(0, 0, 0);
   $ctx->stroke();
 }
 
+=head2 draw_UTC
+
+=cut
+
 sub draw_UTC
 {
-  my ($ctx, $obj, $pa) = @_;
+  my ($ctx, $obj, $pa, $xywref) = @_;
   my $feat = $obj->{feat};
   my $radius = 50;
   my $UTCHeight = 50;
+ 
   my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
   $ctx->set_source_rgb($colref->[0], $colref->[1], $colref->[2]);
-  my ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
-  my $width = $end - $start;
-  #inital the drawing by identify which direction is processing and move to the start point
-  if ($feat->strand == 1)
+ 
+  my ($start, $end, $width, $movey);
+  unless ($xywref)
   {
-    $ctx->move_to($start, $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT});
+    ($start, $end) = ($obj->{DrawStart}, $obj->{DrawEnd});
+    $width = $end - $start;
+    $movey = $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+    if ($feat->strand == -1)
+    {
+      ($start, $end) = ($end, $start);
+      $movey+= $pa->{STRAND_DISTANCE};
+      my $unitVec = ($width/abs($width));
+      $radius = $radius*$unitVec;
+    }   
   }
-  elsif ($feat->strand == -1)
+  else
   {
-    ($start, $end) = ($end, $start);
-    $ctx->move_to($start, $pa->{SCALE_HEIGHT} + $obj->{LevelNum} * $pa->{LEVEL_HEIGHT} + $pa->{STRAND_DISTANCE});
-    my $unitVec = ($width/abs($width));
-    $radius = $radius*$unitVec;
-  }
+    $width = $xywref->[2];
+    $movey = $xywref->[1];
+    ($start, $end) = ($xywref->[0], $xywref->[0] + $width);
+  } 
+  $ctx->move_to($start, $movey);
+ 
   my ($x2, $x3);
   #Calculate x2 = rectangle's Len , x3 = triangle part's Len |----x2----->
   if (abs($width) <= abs($radius*1.3))
@@ -510,51 +656,47 @@ sub draw_UTC
   $ctx->stroke();
 }
 
+=head2 draw_deletion
+
+=cut
+
+sub draw_deletion
+{
+  my ($ctx, $obj, $pa, $xywref) = @_;
+  my $feat = $obj->{feat};
+  my $height = 50;
+  my $width = 200/$pa->{FACTOR};
+  $width = 7 if ($width < 7);
+ 
+  my $colref = $pa->{FEAT_RGB}->{$feat->primary_tag};
+  $ctx->set_source_rgb($colref->[0], $colref->[1], $colref->[2]);
+ 
+  my ($start, $movey);
+  unless ($xywref)
+  {
+    $start = $obj->{DrawStart};
+    my $ypos = $obj->{LevelNum} * $pa->{LEVEL_HEIGHT};
+    $movey =  $pa->{STRAND_Y_POS} + $pa->{STRAND_DISTANCE} + $ypos;
+  }
+  else
+  {   
+    $start = $xywref->[0];
+    $movey = $xywref->[1];
+  } 
+  $ctx->move_to($start - $width / 2, $movey);
+
+  $ctx->rel_line_to($width, -$height);
+  $ctx->rel_move_to(0, $height);
+  $ctx->rel_line_to(-$width, -$height);
+ 
+  $ctx->set_source_rgb(0, 0, 0);
+  $ctx->stroke();
+}
+
 1;
+
 __END__
 
-=head1 NAME
-
-BioStudio::Cairo
-
-=head1 VERSION
-
-Version 1.03
-
-=head1 DESCRIPTION
-
-BioStudio functions for Cairo interaction.
-
-=head1 FUNCTIONS
-
-=head2 draw_scale()
-
-=head2 draw_feature()
-
-=head2 draw_RE()
-
-=head2 draw_stop()
-
-=head2 draw_centromere()
-
-=head2 draw_ARS()
-
-=head2 draw_SSTR()
-
-=head2 draw_CDS()
-
-=head2 draw_intron()
-
-=head2 draw_amplicon()
-
-=head2 draw_repeats()
-
-=head2 draw_UTC()
-
-=head1 AUTHOR
-
-Sarah Richardson <notadoctor@jhu.edu>.
-
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (c) 2011, BioStudio developers
